import React, { useId, useCallback, useEffect,useState } from 'react';
import { useSelector,useDispatch } from 'react-redux';
//import React, { useCallback,useState } from 'react';
import { useMediaQuery } from '@mui/material';
import { useTheme } from '@mui/styles';
import { map } from './core/MapView';
import { formatTime, getStatusColor } from '../common/util/formatter';
import { mapIconKey } from './core/preloadImages';
import { useAttributePreference } from '../common/util/preferences';
import { useCatchCallback } from '../reactHelper';
import { findFonts, fontSize, drawGeofenceBoundingBox,drawPickUpPoints } from './core/mapUtil';
import { useQueue,useSpeech } from "react-text-to-speech";
import { useSpeechSynthesis } from "react-speech-kit";
import { devicesActions } from '../store';
import dimensions from '../common/theme/dimensions';

let vehicles = {};
let lastUpdateMapTimeStamp=Date.now();
let polyPts=[];
let theGeofence;
let focusDeviceId="-1";
let lastAnnoucedTimeTracker={};

const MapPositions = ({ positions, onClick, showStatus, selectedPosition, titleField }) => {
  const id = useId();
  const clusters = `${id}-clusters`;
  const selected = `${id}-selected`;

  const theme = useTheme();
  const desktop = useMediaQuery(theme.breakpoints.up('md'));
  const iconScale = useAttributePreference('iconScale', desktop ? 0.75 : 1);

  const devices = useSelector((state) => state.devices.items);
  const selectedDeviceId = useSelector((state) => state.devices.selectedId);
  const geofences = useSelector((state) => state.geofences.items);
  const state = useSelector((state) => state);

  const mapCluster = useAttributePreference('mapCluster', true);
  const directionType = useAttributePreference('mapDirection', 'selected');

  const geofenceCoordinates = extractCoordinates(geofences);
  //const [vehicles, setVehicles]= useState({});
  //let vehicles={};
  const [announceText, setAnnounceText]= useState("");
  const { speak } = useSpeechSynthesis();
  const dispatch = useDispatch();
  let theText = "Faris"
  let lastAnnounceTimeStamp = 0;
  let annouceTimeInterval = 3000;
  let announceCount = 0;
  let initialAnnounceTime =0;


  function extractCoordinates(geofences){
    var result=[];
    console.log("The geofence");
    console.log(geofences);
    theGeofence = Object.values(geofences)[0];
    console.log(theGeofence);
    if(theGeofence != undefined){
      let coordinatesArr = theGeofence.area.replaceAll("POLYGON","").replaceAll("((","").replaceAll("))","").trim().split(",")
      for(var i=0;i<coordinatesArr.length;i++){
        var ithPt = coordinatesArr[i].trim();
        var ptArr = ithPt.split(" ")
        result.push([parseFloat(ptArr[1].trim()),parseFloat(ptArr[0].trim())]);
        polyPts.push({x:parseFloat(ptArr[1].trim()), y:parseFloat(ptArr[0].trim())})
      }

    }else{
      alert("Geofence perimeter is not configured.");
    }

    return result;
  }

  function focusMapToGeofence(){
    let geofenceBearing=0;
    if(theGeofence!=undefined){
      geofenceBearing=theGeofence.attributes.displayBearing;
    }
    let centroid = get_polygon_centroid(polyPts);
    map.easeTo({
            center: [centroid.x, centroid.y],
            zoom: 18,
            //offset: [0, -dimensions.popupMapOffset / 2],
            bearing:geofenceBearing
          });

  }

  function get_polygon_centroid(pts) {
    var first = pts[0], last = pts[pts.length-1];
    if (first.x != last.x || first.y != last.y) pts.push(first);
    var twicearea=0,
    x=0, y=0,
    nPts = pts.length,
    p1, p2, f;
    for ( var i=0, j=nPts-1 ; i<nPts ; j=i++ ) {
       p1 = pts[i]; p2 = pts[j];
       f = p1.x*p2.y - p2.x*p1.y;
       twicearea += f;          
       x += ( p1.x + p2.x ) * f;
       y += ( p1.y + p2.y ) * f;
    }
    f = twicearea * 3;
    return { x:x/f, y:y/f };
 }

  function isInsidePolygon(point, vs) {
    
    var x = point[0], y = point[1];
    
    var inside = false;
    for (var i = 0, j = vs.length - 1; i < vs.length; j = i++) {
        var xi = vs[i][0], yi = vs[i][1];
        var xj = vs[j][0], yj = vs[j][1];
        
        var intersect = ((yi > y) != (yj > y))
            && (x < (xj - xi) * (y - yi) / (yj - yi) + xi);
        if (intersect) inside = !inside;
    }
    
    return inside;
};

  const createFeature = (devices, position, selectedPositionId) => {
    // TODO: Add auto announce voice HERE
    const device = devices[position.deviceId];
    //alert(device.name+ " " +position+" "+selectedPositionId);
    let showDirection;
    switch (directionType) {
      case 'none':
        showDirection = false;
        break;
      case 'all':
        showDirection = position.course > 0;
        break;
      default:
        showDirection = selectedPositionId === position.id && position.course > 0;
        break;
    }
    let size = selectedPositionId === position.id ? 0: 1;
    return {
      id: position.id,
      deviceId: position.deviceId,
      name: device.name,
      fixTime: formatTime(position.fixTime, 'seconds'),
      category: mapIconKey(device.category),
      color: showStatus ? position.attributes.color || getStatusColor(device.status) : 'neutral',
      rotation: position.course,
      direction: showDirection,
      size: size,
    };
  };

  const onMouseEnter = () => map.getCanvas().style.cursor = 'pointer';
  const onMouseLeave = () => map.getCanvas().style.cursor = '';

  const onMapClick = useCallback((event) => {
    if (!event.defaultPrevented && onClick) {
      const min = 1;
      const max = 100;
      const rand = min + Math.random() * (max - min);
      let focusDiv = document.getElementById("focusDiv");
         if(focusDiv != null){
           document.getElementById("focusDiv").style.zIndex="-1";
         }
         focusMapToGeofence();
      //announceArrival(theText+parseInt(rand).toString()+" is here.")
      onClick(event.lngLat.lat, event.lngLat.lng);
    }
  }, [onClick]);
  

  function announceArrival(deviceId){
      focusDeviceId="2";
      let currentTimeStamp = Date.now()
        if(announceCount==0){
          lastAnnounceTimeStamp=currentTimeStamp;
          initialAnnounceTime=currentTimeStamp
        }
        announceCount+=1;
        let safeAnnounceTime = initialAnnounceTime + (announceCount*annouceTimeInterval);
        let timeDiff = (currentTimeStamp - lastAnnounceTimeStamp) > (safeAnnounceTime - lastAnnounceTimeStamp) ? (0) : (safeAnnounceTime - currentTimeStamp);
        lastAnnounceTimeStamp=currentTimeStamp + timeDiff
        console.log(deviceId);
        setTimeout(function() {   
          console.log("Annoucing Arrival..");
          const device = devices[deviceId];

          if(window.START_CLICKED)
            dispatch(devicesActions.selectId(deviceId));
          else
            focusMapToGeofence();
          console.log(device);
          
          const min = 1;
          const max = 10;
          const rand = min + Math.random() * (max - min);
          const rand2 = min + Math.random() * (max - min);
        let message = (new Array(parseInt(rand)).join(' '))+device.name.split("").join(" ") + device.contact+(new Array(parseInt(rand2)).join(' '));
        setAnnounceText(message);
        }, timeDiff);
  }



  const onMarkerClick = useCallback((event) => {
    event.preventDefault();
    const feature = event.features[0];
    if (onClick) {
      onClick(feature.properties.id, feature.properties.deviceId);
    }
  }, [onClick]);

  useEffect(() => {
    start();
}, [announceText])

  const {
    Text, // Component that returns the modified text property
    speechStatus, // String that stores current speech status
    isInQueue, // Indicates whether the speech is currently playing or waiting in the queue
    start, // Function to start the speech or put it in queue
    pause, // Function to pause the speech
    stop // Function to stop the speech or remove it from queue
  } = useSpeech({ text: announceText ,rate:1.5, preserveUtteranceQueue: true,
    onError: (error) => {
      alert(error.message);
    },
    onStart: (event) => {
      console.log("Speech Started:", event);
    },
    onResume: (event) => {
      console.log("Speech Resumed:", event);
    },
    onPause: (event) => {
      console.log("Speech Paused:", event);
    },
    onStop: (event) => {
      console.log("Speech Stopped:", event);
     

    },
    onBoundary: (event) => {
      console.log("Boundary:", event);
    },
    onQueueChange: (queue) => {
      console.log("Queue updated:", queue);
    },
  });

  const onClusterClick = useCatchCallback(async (event) => {
    event.preventDefault();
    const features = map.queryRenderedFeatures(event.point, {
      layers: [clusters],
    });
    const clusterId = features[0].properties.cluster_id;
    const zoom = await map.getSource(id).getClusterExpansionZoom(clusterId);
    map.easeTo({
      center: features[0].geometry.coordinates,
      zoom
    });
  }, [clusters]);

  useEffect(() => {
    map.addSource(id, {
      type: 'geojson',
      data: {
        type: 'FeatureCollection',
        features: [],
      },
      cluster: mapCluster,
      clusterMaxZoom: 14,
      clusterRadius: 50,
    });
    map.addSource(selected, {
      type: 'geojson',
      data: {
        type: 'FeatureCollection',
        features: [],
      },
    });
    [id, selected].forEach((source) => {
      map.addLayer({
        id: source,
        type: 'symbol',
        source,
        filter: ['!has', 'point_count'],
        layout: {
          'icon-image': '{category}-{color}',
          'icon-size': [
            'match', 
            ['get', 'size'], // Use the result 'nation' property
            0, 1.5,
            1, iconScale,
            iconScale // any other nation
            ],
          'icon-allow-overlap': true,
          'text-field': `{${titleField || 'name'}}`,
          'text-allow-overlap': true,
          'text-anchor': 'bottom',
          'text-offset': [0, -2 * iconScale],
          'text-font': findFonts(map),
          'text-size': 12,
        },
        paint: {
          'text-halo-color': `white`,
          'text-halo-width': 1,
        },
      });
      map.addLayer({
        id: `direction-${source}`,
        type: 'symbol',
        source,
        filter: [
          'all',
          ['!has', 'point_count'],
          ['==', 'direction', true],
        ],
        layout: {
          'icon-image': 'direction',
          'icon-size': iconScale,
          'icon-allow-overlap': true,
          'icon-rotate': ['get', 'rotation'],
          'icon-rotation-alignment': 'map',
        },
      });

      map.on('mouseenter', source, onMouseEnter);
      map.on('mouseleave', source, onMouseLeave);
      map.on('click', source, onMarkerClick);
    });
    map.addLayer({
      id: clusters,
      type: 'symbol',
      source: id,
      filter: ['has', 'point_count'],
      layout: {
        'icon-image': 'background',
        'icon-size': iconScale,
        'text-field': '{point_count_abbreviated}',
        'text-font': findFonts(map),
        'text-size': 14,
      },
    });

    map.on('mouseenter', clusters, onMouseEnter);
    map.on('mouseleave', clusters, onMouseLeave);
    map.on('click', clusters, onClusterClick);
    map.on('click', onMapClick);
    

    return () => {
      map.off('mouseenter', clusters, onMouseEnter);
      map.off('mouseleave', clusters, onMouseLeave);
      map.off('click', clusters, onClusterClick);
      map.off('click', onMapClick);

      if (map.getLayer(clusters)) {
        map.removeLayer(clusters);
      }

      [id, selected].forEach((source) => {
        map.off('mouseenter', source, onMouseEnter);
        map.off('mouseleave', source, onMouseLeave);
        map.off('click', source, onMarkerClick);

        if (map.getLayer(source)) {
          map.removeLayer(source);
        }
        if (map.getLayer(`direction-${source}`)) {
          map.removeLayer(`direction-${source}`);
        }
        if (map.getSource(source)) {
          map.removeSource(source);
        }
      });
    };
  }, [mapCluster, clusters, onMarkerClick, onClusterClick,selectedPosition]);


  useEffect(() => {
    [id, selected].forEach((source) => {
        map.getSource(source)?.setData({
          type: 'FeatureCollection',
          features: positions.filter((it) => devices.hasOwnProperty(it.deviceId))
            .filter((it) => (source === id ? it.deviceId !== selectedDeviceId : it.deviceId === selectedDeviceId))
            .map((position) => ({
              type: 'Feature',
              geometry: {
                type: 'Point',
                coordinates: [position.longitude, position.latitude],
              },
              properties: createFeature(devices, position, selectedPosition && selectedPosition.id),
            })),
        });
    });
  }, [mapCluster, clusters, onMarkerClick, onClusterClick, devices, positions, selectedPosition]);

  useEffect(() => {
    let currentTimeStamp=Date.now()
    console.log(currentTimeStamp-lastUpdateMapTimeStamp);
    //if(currentTimeStamp-lastUpdateMapTimeStamp > 2000){
    
    if(window.START_CLICKED){
      document.getElementById('oneTouchDiv').style.zIndex="0";
      console.log(positions);
      console.log("Start Calculating:"+positions.length)
      for(var i=0; i<positions.length; i++){
        var ithPos = positions[i];
        var existingVehicle = vehicles[ithPos.deviceId]
        console.log("Vehicles position in geofence:");
        console.log(vehicles);

        if( typeof existingVehicle !=="undefined")
          console.log(existingVehicle.latitude +" vs "+ithPos.latitude);

        if( typeof existingVehicle ==="undefined" || existingVehicle.latitude !== ithPos.latitude || existingVehicle.longitude !== ithPos.longitude){
        //if(true){
          vehicles[ithPos.deviceId.toString()]=ithPos;
          console.log(ithPos.latitude);
          let pt = [ithPos.longitude, ithPos.latitude];
          let isInside = isInsidePolygon(pt, geofenceCoordinates);
          console.log(isInside);
          console.log(state);

          if(isInside){

              console.log("Time Diff:"+(currentTimeStamp-lastAnnoucedTimeTracker[ithPos.deviceId])+"s");

              //auto announced every 60s when vehicle inside geofence
            if(!lastAnnoucedTimeTracker.hasOwnProperty(ithPos.deviceId.toString())|| (currentTimeStamp-lastAnnoucedTimeTracker[ithPos.deviceId.toString()] > 60000)){
              announceArrival(ithPos.deviceId);
              lastAnnoucedTimeTracker[ithPos.deviceId.toString()] = currentTimeStamp;
            }
          }else{
            console.log("vehicles to be deleted:");
            console.log(vehicles[ithPos.deviceId]);
            delete vehicles[ithPos.deviceId];
          }

        }
      };
      //vehicles={}
      console.log("time tracker:");
      console.log(lastAnnoucedTimeTracker);
    }
    lastUpdateMapTimeStamp=currentTimeStamp
  }, [positions]);

  useEffect(() =>{
    drawPickUpPoints(map,geofences,theGeofence.id);
    drawGeofenceBoundingBox(map,geofences,theGeofence.id);
  },[]
  )


  return null;
};

export default MapPositions;